#define _CRT_SECURE_NO_WARNINGS
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>


#ifndef M_PI
#define M_PI (3.14159265358979323846264338327950288)
#endif

typedef struct
{
	double x;
	double y;
	double z;
} point_t;

typedef struct
{
	point_t	point[2048];
	int x;
	int y;
	int heading;
} laserscan_t;

char *get_file(char *filename)
{
	FILE	*file;
	char	*buffer;
	int	file_size, bytes_read;

	file = fopen(filename, "rb");
	if (file == NULL)
		return 0;
	fseek(file, 0, SEEK_END);
	file_size = ftell(file);
	fseek(file, 0, SEEK_SET);
	buffer = (char *)malloc(file_size * sizeof(char) + 1);
	bytes_read = (int)fread(buffer, sizeof(char), file_size, file);
	if (bytes_read != file_size)
		return 0;
	fclose(file);
	buffer[file_size] = '\0';
	return buffer;
}

int load_laserscan(laserscan_t *scan, char *filename, int step)
{
	FILE *fp = fopen(filename, "r");
	if (fp == NULL)
	{
		perror("Unable to open file");
		return -1;
	}

	int num_point = 0;

	char line[4096] = { 0 };
	char *pline = &line[0];
	int i = 0;

	while (fgets(line, 4096, fp) != NULL)
	{
		char *pline = &line[0];
		//printf("%s\n", line);

		// skip non data lines
		if (strstr(line, "theta") == NULL)
			continue;

		if (strstr(line, "Do you want to see all the data? (y/n)") != NULL)
		{
			pline += strlen("Do you want to see all the data? (y/n) ");
		}


		float angle, distance, quality;
	//theta: 0.37 Dist : 02656.00
		if (2 == sscanf(pline, "   theta: %f Dist: %f ", &angle, &distance))
		{
			// convert to cartesian from radians
			double cos_v = cos(angle * (0.0174533));
			double sin_v = sin(angle * (0.0174533));


			/*
			temp = vert->position.y;
			vert->position.y = vert->position.z;
			vert->position.z =  -temp;
			*/


			scan->point[i].x = distance * cos_v;
			scan->point[i].y = distance * sin_v;
			scan->point[i].z = 0.0;
			i++;
			num_point++;
			if (i >= 2048)
			{
				// filled up, most have 1080, one with 1096
				break;
			}

		}
	}


	double pitch = M_PI / 6.0;
	double roll = 0.0;
	double yaw = (step * 1.8) * (-M_PI / 180.0);

	double cosa = cos(yaw);
	double sina = sin(yaw);

	double cosb = cos(pitch);
	double sinb = sin(pitch);

	float cosc = cos(roll);
	float sinc = sin(roll);

	double Axx = cosa*cosb;
	double Axy = cosa*sinb*sinc - sina*cosc;
	double Axz = cosa*sinb*cosc + sina*sinc;

	double Ayx = sina*cosb;
	double Ayy = sina*sinb*sinc + cosa*cosc;
	double Ayz = sina*sinb*cosc - cosa*sinc;

	double Azx = -sinb;
	double Azy = cosb*sinc;
	double Azz = cosb*cosc;

	for (int i = 0; i < num_point; i++)
	{
		float px = scan->point[i].x;
		float py = scan->point[i].y;
		float pz = scan->point[i].z;

		scan->point[i].x = Axx*px + Axy*py + Axz*pz;
		scan->point[i].y = Ayx*px + Ayy*py + Ayz*pz;
		scan->point[i].z = Azx*px + Azy*py + Azz*pz;

		printf("%lf, %lf, %lf,\n", scan->point[i].x, scan->point[i].y, scan->point[i].z);
	}

}
laserscan_t scan[128];

int main(int argc, char *argv[])
{
	int start = atoi(argv[1]);
	int end = atoi(argv[2]);

	char file[128] = { 0 };
	for (int i = start; i < end; i++)
	{
		sprintf(file, "scan%d", i + 1);
		load_laserscan(&scan[i], file, i);
	}


	return 0;
}